import unittest
import uuid

from requests import PreparedRequest, Response, Session
from unittest.mock import patch, Mock

from identixone.http.client import IdentixOneHttpClient


class TestHttpClientRequest(unittest.TestCase):

    def setUp(self):
        self.session_patch = patch('identixone.http.client.Session')

        self.request_mock = Mock(wraps=PreparedRequest())
        self.request_mock.headers = {}

        self.session_mock = Mock(wraps=Session())
        self.session_mock.prepare_request.return_value = self.request_mock
        self.session_mock.send.return_value = Response()
        self.session_mock.headers = {}

        session_constructor_mock = self.session_patch.start()
        session_constructor_mock.return_value = self.session_mock

        self.auth_token = uuid.uuid4().hex

        self.client = IdentixOneHttpClient(auth_token=self.auth_token)

    def tearDown(self):
        self.session_patch.stop()

    def test_request_sets_common_headers(self):
        self.client.request('method', 'URL')
        assert self.client.session.headers['User-Agent'].startswith(
            'identixone-python')
        self.assertIsNotNone(self.client.session.headers['Request-ID'])

    def test_set_accept_header_if_missing(self):
        self.client.request('method', 'URL')
        self.assertEqual(
            'application/json', self.client.session.headers['Accept'])

    def test_specify_accept_header_takes_effect(self):
        self.client.request('method', 'URL', headers={'Accept': 'text/html'})
        self.assertEqual('text/html', self.client.session.headers['Accept'])



import json
import unittest
import uuid

from requests import Response, Session
from unittest.mock import patch

from identixone.api.client import Client
from identixone.base.choices import (
    Conf, NotificationHTTPMethod, NotificationTransport)


class TestAPIModule(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        self.version = 1
        self.token = uuid.uuid4().hex
        self.client = Client('token', version=1)
        super(TestAPIModule, self).__init__(*args, **kwargs)

    @staticmethod
    def response(resp_body, status_code):
        response = Response()
        response.status_code = status_code
        response._content = json.dumps(resp_body).encode('utf-8')
        return response


class TestAPIAuthModule(TestAPIModule):

    @patch.object(Session, 'send')
    def test_create_temp_token(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)

        response = self.client.auth.create_token(permanent=False)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

        response = self.client.auth.create_token(permanent=True)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)


class TestAPINotificationsModule(TestAPIModule):

    @patch.object(Session, 'send')
    def test_list_notifications(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.notifications.list()
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_get_notifications(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.notifications.get(notification_id=1)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_create_notifications(self, mocked_send):
        status_code = 201
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.notifications.create(
            name='name', is_active=True,
            transport=NotificationTransport.WEBHOOK,
            http_method=NotificationHTTPMethod.GET,
            conf_thresholds=[Conf.EXACT, Conf.HA, Conf.JUNK],
            destination_url='https://api.identix.one/')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_update_notifications(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.notifications.update(
            notification_id=1, is_active=False)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_delete_notifications(self, mocked_send):
        status_code = 204
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.notifications.delete(notification_id=1)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)


class TestAPIPersonsModule(TestAPIModule):

    @patch.object(Session, 'send')
    def test_create_persons(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.persons.create(photo='photo', source='source')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_search_persons(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.persons.search(photo='photo')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_reinit_image_persons(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.persons.reinit_image(
            idxid='1', photo='photo', source='source', conf=Conf.EXACT)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_reinit_id_persons(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.persons.reinit_id(id=1)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_delete_persons(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.persons.delete(idxid='1')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)


class TestAPIRecordsModule(TestAPIModule):

    @patch.object(Session, 'send')
    def test_list_records(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.records.list(
            new=True, junk=False, ha=False, qty=10)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_get_records(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.records.get(idxid='1')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_entry_delete_records(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.records.entry_delete(entry_id=1)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)


class TestAPISourcesModule(TestAPIModule):

    @patch.object(Session, 'send')
    def test_list_sources(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.sources.list()
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_get_sources(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.sources.get(source_id=1)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_create_sources(self, mocked_send):
        status_code = 201
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.sources.create(name='name')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_update_sources(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.sources.update(source_id=1, name='other')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_delete_sources(self, mocked_send):
        status_code = 204
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.sources.delete(source_id=1)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)


class TestAPIUsersModule(TestAPIModule):

    @patch.object(Session, 'send')
    def test_users_me(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.me()
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_users_change_password(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.change_password(
            password='p', password2='p', reset_tokens=False)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_users_list_tokens(self, mocked_send):
        status_code = 200
        resp_body = [{'key': 'value'}]
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.list_tokens(permanent=False)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

        resp_body = []
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.list_tokens(permanent=True)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

        resp_body = [{'key2': 'value2'}]
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.list_tokens()
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_users_bulk_delete_tokens(self, mocked_send):
        status_code = 204
        resp_body = None
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.bulk_delete()
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

        response = self.client.users.bulk_delete(permanent=False)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

        response = self.client.users.bulk_delete(permanent=True)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_users_get_tokens(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.get_token(id_or_token='id')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_users_update_tokens(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.update_token(
            id_or_token='id', is_active=False)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_users_delete_tokens(self, mocked_send):
        status_code = 204
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.users.delete_token(id_or_token='id')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)


class TestAPIUtilityModule(TestAPIModule):

    @patch.object(Session, 'send')
    def test_utility_asm(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.utility.asm(photo='photo')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_utility_liveness(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.utility.liveness(photo='photo')
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_utility_compare(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.utility.compare(
            photo1='photo1', photo2='photo2',
            liveness_photo1=False, liveness_photo2=False,
            conf=Conf.JUNK)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)

    @patch.object(Session, 'send')
    def test_utility_customer(self, mocked_send):
        status_code = 200
        resp_body = {'key': 'value'}
        mocked_send.return_value = self.response(resp_body, status_code)
        response = self.client.utility.customer(source='source', offset=20)
        self.assertEqual(response.json(), resp_body)
        self.assertEqual(response.status_code, status_code)



class NotificationTransport(object):
    WEBHOOK = 0
    WEBSOCKETS_CLIENT = 1
    WEBSOCKETS_SERVER = 2


class NotificationHTTPMethod(object):
    POST = 0
    GET = 1


class Conf(object):
    NEW = 'new'
    EXACT = 'exact'
    HA = 'ha'
    JUNK = 'junk'
    NM = 'nm'
    DET = 'det'
    REINIT = 'reinit'



class Users(object):

    def __init__(self, http_client):
        self.http_client = http_client

    def me(self):
        return self.http_client.get('v1/users/me/')

    def change_password(self, password, password2, reset_tokens=True):
        data = {
            'password': password,
            'password2': password2,
            'reset_tokens': reset_tokens
        }
        return self.http_client.post('v1/users/password/change/', data=data)

    def list_tokens(self, permanent=None):
        params = None
        if permanent is not None:
            params = {'permanent': permanent}
        return self.http_client.get('v1/users/tokens/', params=params)

    def get_token(self, id_or_token):
        return self.http_client.get(
            'v1/users/tokens/{}/'.format(id_or_token))

    def update_token(self, id_or_token, is_active=True):
        data = {'is_active': is_active}
        return self.http_client.patch(
            'v1/users/tokens/{}/'.format(id_or_token), data=data)

    def delete_token(self, id_or_token):
        return self.http_client.delete(
            'v1/users/tokens/{}/'.format(id_or_token))

    def bulk_delete(self, permanent=None):
        params = None
        if permanent is not None:
            params = {'permanent': permanent}
        return self.http_client.delete('v1/users/tokens/', params=params)



from identixone.base.choices import Conf


class Sources(object):

    def __init__(self, http_client):
        self.http_client = http_client

    def list(self):
        return self.http_client.get('v1/sources/')

    def get(self, source_id):
        return self.http_client.get('v1/sources/{}/'.format(source_id))

    def create(self, name, pps_timestamp=False,
               identify_facesize_threshold=None, auto_create_persons=False,
               auto_create_facesize_threshold=None, auto_create_on_ha=False,
               auto_create_on_junk=False, auto_check_face_angle=True,
               auto_check_angle_threshold=None, auto_check_asm=True,
               auto_create_check_blur=True, auto_create_check_exp=True,
               auto_check_liveness=False, auto_create_liveness_only=False,
               manual_create_facesize_threshold=None,
               manual_create_on_ha=False, manual_create_on_junk=False,
               manual_check_asm=True, manual_create_liveness_only=False,
               manual_check_liveness=False,
               store_images_for_confs=[
                   Conf.NEW, Conf.EXACT, Conf.HA, Conf.JUNK,
                   Conf.NM, Conf.DET, Conf.REINIT]):
        data = {
            'name': name,
            'pps_timestamp': pps_timestamp,
            'identify_facesize_threshold': identify_facesize_threshold,
            'auto_create_persons': auto_create_persons,
            'auto_create_facesize_threshold': auto_create_facesize_threshold,
            'auto_create_on_ha': auto_create_on_ha,
            'auto_create_on_junk': auto_create_on_junk,
            'auto_check_face_angle': auto_check_face_angle,
            'auto_check_angle_threshold': auto_check_angle_threshold,
            'auto_check_asm': auto_check_asm,
            'auto_create_check_blur': auto_create_check_blur,
            'auto_create_check_exp': auto_create_check_exp,
            'auto_check_liveness': auto_check_liveness,
            'auto_create_liveness_only': auto_create_liveness_only,
            'manual_create_facesize_threshold': manual_create_facesize_threshold,  # noqa: E501
            'manual_create_on_ha': manual_create_on_ha,
            'manual_create_on_junk': manual_create_on_junk,
            'manual_check_asm': manual_check_asm,
            'manual_create_liveness_only': manual_create_liveness_only,
            'manual_check_liveness': manual_check_liveness,
            'store_images_for_confs': store_images_for_confs
        }
        return self.http_client.post('v1/sources/', data=data)

    def update(self, source_id, **kwargs):
        return self.http_client.patch(
            'v1/sources/{}/'.format(source_id), data=kwargs)

    def delete(self, source_id):
        return self.http_client.delete('v1/sources/{}/'.format(source_id))



import os
import unittest

from identixone.base.exceptions import ImproperlyConfigured
from identixone.utils.environment import env_var, ENV_BASE_PREFIX


class TestEnvironmentVar(unittest.TestCase):

    def test_os_environ_valid_var(self):
        key = 'KEY'
        value = 'VALUE'
        os.environ['{}{}'.format(ENV_BASE_PREFIX, key)] = value
        self.assertEqual(env_var(os.environ, key), value)

    def test_os_environ_invalid_without_prefix(self):
        key = 'KEY'
        value = 'VALUE'
        os.environ[key] = value
        self.assertIsNone(env_var(os.environ, key))

    def test_dict_environ_valid_var(self):
        environ = {}
        key = 'KEY'
        value = 'VALUE'
        environ['{}{}'.format(ENV_BASE_PREFIX, key)] = value
        self.assertEqual(env_var(environ, key), value)

    def test_dict_environ_invalid_without_prefix(self):
        environ = {}
        key = 'KEY'
        value = 'VALUE'
        environ[key] = value
        self.assertIsNone(env_var(os.environ, key))

    def test_invalid_environ_raises_exception(self):
        key = 'KEY'
        with self.assertRaises(ImproperlyConfigured):
            env_var([], key)



import platform
import uuid

from requests import Request, Session, hooks

from urllib.parse import urljoin

from identixone import __version__
from identixone.http.auth import IdentixOneAuth
from identixone.utils import constants


class IdentixOneHttpClient(object):
    """
    Http client used to make actual HTTP requests to Identix.one API.
    You can subclass it and provide new class as http_client param into Client.
    """

    def __init__(self, auth_token, request_hooks=None, timeout=None):
        self.auth = IdentixOneAuth(token=auth_token)
        self.session = Session()
        self.request_hooks = request_hooks or hooks.default_hooks()
        self.timeout = timeout or constants.HTTP_CLIENT_TIMEOUT
        self.base_url = constants.HTTP_CLIENT_BASE_URL

    @property
    def common_headers(self):
        return {
            'User-Agent': 'identixone-python/{} (Python {})'.format(
                __version__, platform.python_version()),
            'Request-ID': str(uuid.uuid4())
        }

    def get(self, url, params=None, data=None, files=None,
            headers=None, auth=None):
        return self.request(
            method='GET', url=url, params=params,
            data=data, files=files, headers=headers, auth=auth)

    def post(self, url, params=None, data=None, files=None,
             headers=None, auth=None):
        return self.request(
            method='POST', url=url, params=params,
            data=data, files=files, headers=headers, auth=auth)

    def patch(self, url, params=None, data=None, files=None,
              headers=None, auth=None):
        return self.request(
            method='PATCH', url=url, params=params,
            data=data, files=files, headers=headers, auth=auth)

    def delete(self, url, params=None, data=None, files=None,
               headers=None, auth=None):
        return self.request(
            method='DELETE', url=url, params=params,
            data=data, files=files, headers=headers, auth=auth)

    def request(self, method, url, params=None, data=None, files=None,
                headers=None, auth=None):
        method = method.upper()
        full_url = urljoin(self.base_url, url)
        headers = headers or {}
        headers.update(self.common_headers)

        if 'Accept' not in headers:
            headers['Accept'] = 'application/json'

        self.session.auth = auth or self.auth
        self.session.headers.update(headers)

        request = Request(**{
            'method': method,
            'url': full_url,
            'params': params,
            'data': data,
            'files': files,
            'headers': headers,
            'hooks': self.request_hooks
        })

        prepped_request = self.session.prepare_request(request)

        return self.session.send(
            prepped_request,
            allow_redirects=False,
            timeout=self.timeout,
        )



from identixone.base.choices import Conf


class Persons(object):

    def __init__(self, http_client):
        self.http_client = http_client

    def create(self, photo, source, facesize=None, create_on_ha=False,
               create_on_junk=False, asm=False, liveness=False,
               create_liveness_only=False):
        data = {
            'source': source,
            'facesize': facesize,
            'create_on_ha': create_on_ha,
            'create_on_junk': create_on_junk,
            'asm': asm,
            'liveness': liveness,
            'create_liveness_only': create_liveness_only
        }
        files = {'photo': photo}
        return self.http_client.post('v1/persons/', data=data, files=files)

    def search(self, photo, asm=False, liveness=False):
        data = {'asm': asm, 'liveness': liveness}
        files = {'photo': photo}
        return self.http_client.post(
            'v1/persons/search/', data=data, files=files)

    def reinit_image(self, idxid, photo, source, conf=Conf.HA, facesize=None,
                     liveness=False, reinit_liveness_only=False):
        data = {
            'source': source,
            'conf': conf,
            'facesize': facesize,
            'liveness': liveness,
            'reinit_liveness_only': reinit_liveness_only
        }
        files = {'photo': photo}
        return self.http_client.post(
            'v1/persons/reinit/{}/'.format(idxid), data=data, files=files)

    def reinit_id(self, id, facesize=None):
        data = {'id': id, 'facesize': facesize}
        return self.http_client.post('v1/persons/reinit/', data=data)

    def delete(self, idxid):
        return self.http_client.delete('v1/persons/{}/'.format(idxid))



class Records(object):

    def __init__(self, http_client):
        self.http_client = http_client

    def list(self, **kwargs):
        # Possible query params:
        #   period_start, period_end, pgn_start, qty,
        #   source, new, reinit, exact, ha, junk, nm, det
        boolean = ['new', 'reinit', 'exact', 'ha', 'junk', 'nm', 'det']
        for k, v in kwargs.items():
            if k in boolean:
                kwargs[k] = str(v).lower()
        return self.http_client.get('v1/records/', params=kwargs)

    def get(self, idxid, **kwargs):
        return self.http_client.get(
            'v1/records/{}/'.format(idxid), params=kwargs)

    def entry_delete(self, entry_id):
        return self.http_client.delete('v1/records/entry/{}/'.format(entry_id))



from identixone.base.exceptions import ImproperlyConfigured

ENV_BASE_PREFIX = 'IDENTIXONE_'


def env_var(environment, name):
    try:
        return environment.get('{}{}'.format(ENV_BASE_PREFIX, name), None)
    except Exception as e:
        raise ImproperlyConfigured('Invalid environment', e)



import os
import inspect

from importlib import import_module

from identixone.base.exceptions import ImproperlyConfigured
from identixone.http.client import IdentixOneHttpClient
from identixone.utils import constants
from identixone.utils.environment import env_var


class Client(object):
    """
    High-level client to interact with the Identix.one API.
    """

    def __init__(self, token=None, version=None,
                 http_client=None, environment=None):
        """
        Initialize Client with credentials and optional http client.

        :param token: Access token to the Identix.one API
        :param version: Version of the API to use
        :param http_client: Class of http client to use for making requests
        :param environment: Environment from which to read config variables
        """
        self.environment = environment or os.environ
        self.token = token or self.env_var('TOKEN')

        self.version = str(version) if version else self.env_var('VERSION')
        if not self.version:
            raise ImproperlyConfigured(
                'Version must be provided. Valid choices are: {}'.format(
                    str(constants.SUPPORTED_API_VERSIONS)))

        if self.version not in constants.SUPPORTED_API_VERSIONS:
            raise ImproperlyConfigured(
                'Invalid version. Valid choices are: {}'.format(
                    str(constants.SUPPORTED_API_VERSIONS)))

        if not self.token:
            raise ImproperlyConfigured('API token must be provided.')

        if http_client:
            if not inspect.isclass(http_client):
                raise ImproperlyConfigured(
                        'Variable http_client has to be class type')
            self.http_client = http_client(auth_token=self.token)
        else:
            self.http_client = IdentixOneHttpClient(auth_token=self.token)

    def dynamic_import(self, module_path, attribute):
        """
        Imports any attribute from the module specified as string dotted path.
        Takes into account current supplied version to the Client instance.

        :param module_path: dotted path of the module from which to import from
        :param attribute: function, class or any other attr to be imported
        :return: imported attribute
        """
        version_str = 'v{}'.format(self.version)
        abs_path = '{}.{}'.format(module_path, version_str)
        module_object = import_module(abs_path)
        return getattr(module_object, attribute)

    def env_var(self, name):
        """
        Wrapper around env_var utility function with supplied current environ.

        :param name: name of the environment variable
        :return: value of the environment variable or None if it doesn't exist
        """
        return env_var(self.environment, name)

    @property
    def auth(self):
        cls = self.dynamic_import('identixone.api.auth', 'Auth')
        return cls(self.http_client)

    @property
    def notifications(self):
        cls = self.dynamic_import(
            'identixone.api.notifications', 'Notifications')
        return cls(self.http_client)

    @property
    def persons(self):
        cls = self.dynamic_import('identixone.api.persons', 'Persons')
        return cls(self.http_client)

    @property
    def records(self):
        cls = self.dynamic_import('identixone.api.records', 'Records')
        return cls(self.http_client)

    @property
    def sources(self):
        cls = self.dynamic_import('identixone.api.sources', 'Sources')
        return cls(self.http_client)

    @property
    def users(self):
        cls = self.dynamic_import('identixone.api.users', 'Users')
        return cls(self.http_client)

    @property
    def utility(self):
        cls = self.dynamic_import('identixone.api.utility', 'Utility')
        return cls(self.http_client)



class Notifications(object):

    def __init__(self, http_client):
        self.http_client = http_client

    def list(self):
        return self.http_client.get('v1/settings/notifications/')

    def create(self, name, is_active, transport, destination_url=None,
               conf_thresholds=None, age_from=None, age_to=None, sex=None,
               moods=None, liveness=None, sources=None,
               http_method=None):
        data = {
            'name': name,
            'is_active': is_active,
            'transport': transport,
            'destination_url': destination_url,
            'conf_thresholds': conf_thresholds,
            'age_from': age_from,
            'age_to': age_to,
            'sex': sex,
            'moods': moods,
            'liveness': liveness,
            'sources': sources,
            'http_method': http_method
        }
        return self.http_client.post('v1/settings/notifications/', data=data)

    def get(self, notification_id):
        return self.http_client.get(
            'v1/settings/notifications/{}/'.format(notification_id))

    def update(self, notification_id, **kwargs):
        return self.http_client.patch(
            'v1/settings/notifications/{}/'.format(notification_id),
            data=kwargs)

    def delete(self, notification_id):
        return self.http_client.delete(
            'v1/settings/notifications/{}/'.format(notification_id))



class Auth(object):

    def __init__(self, http_client):
        self.http_client = http_client

    def create_token(self, permanent=False):
        if permanent:
            url = 'v1/login/permanent/'
        else:
            url = 'v1/login/'
        return self.http_client.post(url)



class IdentixOneException(Exception):
    """Base for all errors produced by this library"""
    pass


class ImproperlyConfigured(IdentixOneException):
    """Errors that occur during configuration of Client"""
    pass



import unittest
import uuid
import os

from identixone.api.client import Client
from identixone.base.exceptions import IdentixOneException


class TestAPIClient(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        self.token = uuid.uuid4().hex
        super(TestAPIClient, self).__init__(*args, **kwargs)

    def test_unsupported_version_raises_exception(self):
        with self.assertRaises(IdentixOneException):
            Client(token=self.token, version=None)

    def test_no_version_raises_exception(self):
        with self.assertRaises(IdentixOneException):
            Client(token=self.token, version=9999)

    def test_no_token_raises_exception(self):
        with self.assertRaises(IdentixOneException):
            Client(token=None, version=1)

    def test_token_get_from_environ(self):
        os.environ['IDENTIXONE_TOKEN'] = '123lskdfjldksjf123'
        client = Client(version='1')
        self.assertEqual(client.token, '123lskdfjldksjf123')

    def test_version_get_from_environ(self):
        os.environ['IDENTIXONE_VERSION'] = '1'
        client = Client(token='adflsdkfjsakldfj')
        self.assertEqual(client.version, '1')

    def test_create_instance_http_client(self):
        class A:
            def __init__(self, auth_token):
                self.token = auth_token

        client = Client(token='sdfdsf',
                        version='1', http_client=A)
        self.assertTrue(isinstance(client.http_client, A))

    def test_http_client_raises_exception(self):
        a = 'http_client'
        with self.assertRaises(IdentixOneException):
            Client(token='sdfdsf', version='1', http_client=a)



from identixone.base.choices import Conf


class Utility(object):

    def __init__(self, http_client):
        self.http_client = http_client

    def asm(self, photo):
        files = {'photo': photo}
        return self.http_client.post('v1/utility/asm/', files=files)

    def liveness(self, photo):
        files = {'photo': photo}
        return self.http_client.post('v1/utility/liveness/', files=files)

    def compare(self, photo1, photo2, liveness_photo1, liveness_photo2,
                conf=Conf.HA):
        files = {'photo1': photo1, 'photo2': photo2}
        data = {
            'liveness_photo1': liveness_photo1,
            'liveness_photo2': liveness_photo2,
            'conf': conf
        }
        return self.http_client.post(
            'v1/utility/compare/', files=files, data=data)

    def customer(self, source, offset=10):
        params = {'source': source, 'offset': offset}
        return self.http_client.get('v1/utility/customer/', params=params)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['requests>=2.0.0']

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="Identix One",
    author_email='dev@identix.one',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python",
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],
    description="A Python package for interacting with the Identix.one API",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords=['identixone', 'identix', 'face', 'recognition', 'cloud'],
    name='identixone',
    packages=find_packages(exclude=['tests', 'tests.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/identixone/identixone-python',
    version='0.1.1',
    zip_safe=False,
)



from requests.auth import AuthBase


class IdentixOneAuth(AuthBase):
    """Attaches Authorization header w/ token to the given Request object."""
    PREFIX = 'Token'

    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers['Authorization'] = '{} {}'.format(self.PREFIX, self.token)
        return r
